#include <iostream>
#include <iomanip>
using namespace std;

// class to store matrices.  Note that this class does not contain
// operators to do arithmetic.  It should!  That is something you learn
// how to do in an object-oriented programming class
class Matrix {
public:
    Matrix();                // default constructor
    Matrix(int nr, int nc);  // constructor - use if you know initial size
    Matrix(const Matrix &orig);                   // copy constructor
    const Matrix & operator=(const Matrix &rhs);  // assigment operator
    virtual ~Matrix();                            // destructor
    bool set(int r, int c, float v);  // set the value of a single element
    void resize(int r, int c);        // change the size of the matrix
    void display(ostream &o) const;   // output the matrix in rows and columns
private:
    int nrows;   // number of rows in the matrix
    int ncols;   // number of columns in the matrix
    float *data; // data should point to an array of floats.  This array
                 // stores all the elements of the array.  Matrix objects
                 // own this memory
};

void Matrix::display(ostream &o) const
{
    for (int i = 0; i < nrows; i++) {
        for (int j = 0; j < ncols; j++)
            o << setw(4) << data[i*ncols+j] << " ";
        o << endl;
    }
}

void Matrix::resize(int r, int c)
{
    if (nrows*ncols != r*c) {
       if (data) delete [] data;
       data = new float[r*c];
    } 
    nrows = r;
    ncols = c;
}

bool Matrix::set(int r, int c, float v)
{
    if (r < 0 || r >= nrows || c < 0 || c >= ncols)
        return false;
    data[r*ncols+c]=v;
    return true;
}

const Matrix & Matrix::operator=(const Matrix &rhs)
{
   if (this != &rhs) {
       if (rhs.ncols*rhs.nrows != nrows*ncols && rhs.nrows*rhs.ncols != 0) {
           if (data) delete [] data;
           data = new float[rhs.nrows*rhs.ncols];
       }
       nrows = rhs.nrows;
       ncols = rhs.ncols;
       for (int i=0; i < nrows*ncols; i++)
           data[i] = rhs.data[i];
   } 
   return *this;
}

Matrix::~Matrix()
{
    if (data) delete [] data;
    data = 0;
    nrows = ncols = 0;
}

Matrix::Matrix(const Matrix &orig)
   : nrows(orig.nrows), ncols(orig.ncols), data(0)
{
   data = new float[nrows*ncols]; 
   for (int i=0; i < nrows*ncols; i++)
       data[i] = orig.data[i];
}

Matrix::Matrix(int nr, int nc)
   : nrows(nr), ncols(nc), data(0)
{
   data = new float[nrows*ncols]; 
}

Matrix::Matrix()
  : nrows(0), ncols(0), data(0)
{
}

ostream & operator<<(ostream& os, const Matrix &m)
{
    m.display(os);
    return os;
}


void foo(Matrix m);

int main()
{
    int i, j;
    Matrix m1;         // statically allocate a Matrix object. Starts out 0x0 
    Matrix m2(3, 4);   // statically allocate a 3x4 matrix object
    for (i=0; i < 3; i++)
        for (j=0; j < 4; j++)
            m2.set(i, j, i*4+j);              // set all values in m2
    Matrix m3(m2);                            // create m3 to be a copy of m2
    m1 = m2 = m2;                             // store m2 in m1
    cout << "Matrix m1, Matrix m2, and Matrix m3: " << endl;
    cout << m1 << endl << m2 << endl << m3 << endl;   // output all 3 matrices

    float v;
    int nr, nc;
    Matrix *mp;          // create a pointer to a matrix
    mp = new Matrix;     // dynamically allocate a matrix
    cout << "Enter the number of rows and columns in your matrix: ";
    cin >> nr >> nc;
    mp->resize(nr, nc);  // set a new size for matrix
    cout << "Enter " << nr*nc << " values for the contents of the array: ";
    for (i=0; i < nr; i++) {
        for (j=0; j < nc; j++) {
            cin >> v;            // input values from the user
            mp->set(i, j, v);    // and store in the matrix
        }
    }
    // output the entire matrix
    cout << "This is your matrix: " << endl;
    cout << *mp << endl;
    foo(*mp);

    delete mp;      // release memory (note: destructor is called automatically)

    return 0;
}

void foo(Matrix m)
{
    cout << "Matrix object passed to a function: " << endl;
    cout << m << endl;
}
